@@ -1,24 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library.
-   Copyright (c) 2022 - Raw Material Software Limited
+   This file is part of the JUCE framework.
+   Copyright (c) Raw Material Software Limited
 
-   JUCE is an open source library subject to commercial or open-source
+   JUCE is an open source framework subject to commercial or open source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
-   Agreement and JUCE Privacy Policy.
+   By downloading, installing, or using the JUCE framework, or combining the
+   JUCE framework with any other source code, object code, content or any other
+   copyrightable work, you agree to the terms of the JUCE End User Licence
+   Agreement, and all incorporated terms including the JUCE Privacy Policy and
+   the JUCE Website Terms of Service, as applicable, which will bind you. If you
+   do not agree to the terms of these agreements, we will not license the JUCE
+   framework to you, and you must discontinue the installation or download
+   process and cease use of the JUCE framework.
 
-   End User License Agreement: www.juce.com/juce-7-licence
-   Privacy Policy: www.juce.com/juce-privacy-policy
+   JUCE End User Licence Agreement: https://juce.com/legal/juce-8-licence/
+   JUCE Privacy Policy: https://juce.com/juce-privacy-policy
+   JUCE Website Terms of Service: https://juce.com/juce-website-terms-of-service/
 
-   Or: You may also use this code under the terms of the GPL v3 (see
-   www.gnu.org/licenses).
+   Or:
 
-   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
-   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
-   DISCLAIMED.
+   You may also use this code under the terms of the AGPLv3:
+   https://www.gnu.org/licenses/agpl-3.0.en.html
+
+   THE JUCE FRAMEWORK IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL
+   WARRANTIES, WHETHER EXPRESSED OR IMPLIED, INCLUDING WARRANTY OF
+   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED.
 
   ==============================================================================
 */
@@ -26,7 +35,7 @@
 namespace juce
 {
 
-struct MPEKeyboardComponent::MPENoteComponent : public Component
+struct MPEKeyboardComponent::MPENoteComponent final : public Component
 {
     MPENoteComponent (MPEKeyboardComponent& o, uint16 sID, uint8 initial, float noteOnVel, float press)
        : owner (o),
@@ -59,7 +68,7 @@ struct MPEKeyboardComponent::MPENoteComponent : public Component
         g.drawEllipse (bounds.withSizeKeepingCentre (pressSize, pressSize), 1.0f);
     }
 
-    //==========================================================================
+    //==============================================================================
     MPEKeyboardComponent& owner;
 
     float radiusScale = 0.0f, noteOnVelocity = 0.0f, pressure = 0.5f;
@@ -100,7 +109,7 @@ void MPEKeyboardComponent::drawWhiteKey (int midiNoteNumber, Graphics& g, Rectan
         auto text = MidiMessage::getMidiNoteName (midiNoteNumber, true, true, getOctaveForMiddleC());
 
         g.setColour (findColour (textLabelColourId));
-        g.setFont (Font (fontHeight).withHorizontalScale (0.8f));
+        g.setFont (withDefaultMetrics (FontOptions { fontHeight }).withHorizontalScale (0.8f));
 
         switch (getOrientation())
         {
@@ -145,7 +154,7 @@ void MPEKeyboardComponent::colourChanged()
     repaint();
 }
 
-//==========================================================================
+//==============================================================================
 MPEValue MPEKeyboardComponent::mousePositionToPitchbend (int initialNote, Point<float> mousePos)
 {
     auto constrainedMousePos = [&]
@@ -378,14 +387,11 @@ void MPEKeyboardComponent::updateNoteComponentBounds (const MPENote& note, MPENo
         const auto currentNote = note.initialNote + (float) note.totalPitchbendInSemitones;
         const auto noteBend = currentNote - std::floor (currentNote);
 
+        const auto averageKeySize = (float) getTotalKeyboardWidth() / (float) (1 + getRangeEnd() - getRangeStart());
+        const auto distance = noteBend * averageKeySize;
+
         const auto noteBounds = getRectangleForKey ((int) currentNote);
-        const auto nextNoteBounds = getRectangleForKey ((int) currentNote + 1);
-
         const auto horizontal = isHorizontal();
-
-        const auto distance = noteBend * (horizontal ? nextNoteBounds.getCentreX() - noteBounds.getCentreX()
-                                                     : nextNoteBounds.getCentreY() - noteBounds.getCentreY());
-
         return (horizontal ? noteBounds.getCentreX() : noteBounds.getCentreY()) + distance;
     }();
 
