@@ -100,12 +100,12 @@ namespace
     Rectangle<Type> coordsToRectangle (Type x, Type y, Type w, Type h) noexcept
     {
        #if JUCE_DEBUG
-        const int maxVal = 0x3fffffff;
+        constexpr int maxVal = 0x3fffffff;
 
-        jassert ((int) x >= -maxVal && (int) x <= maxVal
-              && (int) y >= -maxVal && (int) y <= maxVal
-              && (int) w >= 0 && (int) w <= maxVal
-              && (int) h >= 0 && (int) h <= maxVal);
+        jassertquiet ((int) x >= -maxVal && (int) x <= maxVal
+                   && (int) y >= -maxVal && (int) y <= maxVal
+                   && (int) w >= 0 && (int) w <= maxVal
+                   && (int) h >= 0 && (int) h <= maxVal);
        #endif
 
         return { x, y, w, h };
