@@ -44,7 +44,7 @@
 
   ID:                 juce_graphics
   vendor:             juce
-  version:            8.0.0
+  version:            8.0.3
   name:               JUCE graphics classes
   description:        Classes for 2D vector graphics, image loading/saving, font handling, etc.
   website:            http://www.juce.com/juce
@@ -146,7 +146,6 @@ namespace juce
 #include "fonts/juce_LruCache.h"
 #include "native/juce_RenderingHelpers.h"
 #include "contexts/juce_LowLevelGraphicsSoftwareRenderer.h"
-#include "contexts/juce_LowLevelGraphicsPostScriptRenderer.h"
 #include "effects/juce_ImageEffectFilter.h"
 #include "effects/juce_DropShadowEffect.h"
 #include "effects/juce_GlowEffect.h"
@@ -155,9 +154,3 @@ namespace juce
  #include "native/juce_CoreGraphicsHelpers_mac.h"
  #include "native/juce_CoreGraphicsContext_mac.h"
 #endif
-
-#if JUCE_WINDOWS
- #include "native/juce_Direct2DMetrics_windows.h"
- #include "native/juce_Direct2DGraphicsContext_windows.h"
- #include "native/juce_Direct2DHwndContext_windows.h"
-#endif
