@@ -93,7 +93,7 @@ DECLARE_JNI_CLASS (AndroidAssetManager, "android/content/res/AssetManager")
 #undef JNI_CLASS_MEMBERS
 
 // Defined in juce_core
-std::unique_ptr<InputStream> makeAndroidInputStreamWrapper (jobject stream);
+std::unique_ptr<InputStream> makeAndroidInputStreamWrapper (LocalRef<jobject> stream);
 
 struct AndroidCachedTypeface
 {
