@@ -32,10 +32,6 @@
   ==============================================================================
 */
 
-#include FT_TRUETYPE_TABLES_H
-#include FT_GLYPH_H
-#include FT_COLOR_H
-
 namespace juce
 {
 
@@ -125,7 +121,7 @@ struct FTFaceWrapper final : public ReferenceCountedObject
     MemoryBlock savedFaceData;
     FT_Face face = {};
 
-    JUCE_HEAVYWEIGHT_LEAK_DETECTOR(FTFaceWrapper)
+    JUCE_LEAK_DETECTOR (FTFaceWrapper)
 };
 
 //==============================================================================
@@ -485,7 +481,7 @@ public:
     ~FreeTypeTypeface() override
     {
         if (doCache == DoCache::yes)
-            if (auto* list = FTTypefaceList::getInstance())
+            if (auto* list = FTTypefaceList::getInstanceWithoutCreating())
                 list->removeMemoryFace (ftFace);
     }
 
