@@ -32,7 +32,6 @@
   ==============================================================================
 */
 
-API_AVAILABLE (macos (10.10))
 static void juceFreeAccessibilityPlatformSpecificData (NSAccessibilityElement<NSAccessibility>*)  {}
 
 namespace juce
@@ -45,12 +44,10 @@ class AccessibilityHandler::AccessibilityNativeImpl
 {
 public:
     explicit AccessibilityNativeImpl (AccessibilityHandler& handler)
+        : accessibilityElement (AccessibilityElement::create (handler))
     {
-        if (@available (macOS 10.10, *))
-            accessibilityElement = AccessibilityElement::create (handler);
     }
 
-    API_AVAILABLE (macos (10.10))
     NSAccessibilityElement<NSAccessibility>* getAccessibilityElement() const noexcept
     {
         return accessibilityElement.get();
@@ -58,20 +55,15 @@ public:
 
 private:
     //==============================================================================
-    class API_AVAILABLE (macos (10.10)) AccessibilityElement final : public AccessibleObjCClass<NSAccessibilityElement<NSAccessibility>>
+    class AccessibilityElement final : public AccessibleObjCClass<NSAccessibilityElement<NSAccessibility>>
     {
     public:
         static Holder create (AccessibilityHandler& handler)
         {
-            if (@available (macOS 10.10, *))
-            {
-                static AccessibilityElement cls;
-                Holder element ([cls.createInstance() init]);
-                object_setInstanceVariable (element.get(), "handler", &handler);
-                return element;
-            }
-
-            return {};
+            static AccessibilityElement cls;
+            Holder element ([cls.createInstance() init]);
+            object_setInstanceVariable (element.get(), "handler", &handler);
+            return element;
         }
 
     private:
@@ -825,7 +817,6 @@ private:
     };
 
     //==============================================================================
-    API_AVAILABLE (macos (10.10))
     AccessibilityElement::Holder accessibilityElement;
 
     //==============================================================================
@@ -835,10 +826,7 @@ private:
 //==============================================================================
 AccessibilityNativeHandle* AccessibilityHandler::getNativeImplementation() const
 {
-    if (@available (macOS 10.10, *))
-        return (AccessibilityNativeHandle*) nativeImpl->getAccessibilityElement();
-
-    return nullptr;
+    return (AccessibilityNativeHandle*) nativeImpl->getAccessibilityElement();
 }
 
 static bool areAnyAccessibilityClientsActive()
@@ -872,27 +860,15 @@ static void sendHandlerNotification (const AccessibilityHandler& handler,
     if (! areAnyAccessibilityClientsActive() || notification == NSAccessibilityNotificationName{})
         return;
 
-    if (@available (macOS 10.9, *))
+    if (id accessibilityElement = static_cast<id> (handler.getNativeImplementation()))
     {
-        if (id accessibilityElement = static_cast<id> (handler.getNativeImplementation()))
-        {
-            sendAccessibilityEvent (accessibilityElement, notification,
-                                    (notification == NSAccessibilityLayoutChangedNotification
-                                         ? @{ NSAccessibilityUIElementsKey: @[ accessibilityElement ] }
-                                         : nil));
-        }
+        sendAccessibilityEvent (accessibilityElement, notification,
+                                (notification == NSAccessibilityLayoutChangedNotification
+                                     ? @{ NSAccessibilityUIElementsKey: @[ accessibilityElement ] }
+                                     : nil));
     }
 }
 
-static NSAccessibilityNotificationName layoutChangedNotification()
-{
-    if (@available (macOS 10.9, *))
-        return NSAccessibilityLayoutChangedNotification;
-
-    static NSString* layoutChangedString = @"AXLayoutChanged";
-    return layoutChangedString;
-}
-
 void detail::AccessibilityHelpers::notifyAccessibilityEvent (const AccessibilityHandler& handler, Event eventType)
 {
     auto notification = [eventType]
@@ -901,7 +877,7 @@ void detail::AccessibilityHelpers::notifyAccessibilityEvent (const Accessibility
         {
             case Event::elementCreated:         return NSAccessibilityCreatedNotification;
             case Event::elementDestroyed:       return NSAccessibilityUIElementDestroyedNotification;
-            case Event::elementMovedOrResized:  return layoutChangedNotification();
+            case Event::elementMovedOrResized:  return NSAccessibilityLayoutChangedNotification;
             case Event::focusChanged:           return NSAccessibilityFocusedUIElementChangedNotification;
             case Event::windowOpened:           return NSAccessibilityWindowCreatedNotification;
             case Event::windowClosed:           break;
@@ -925,7 +901,7 @@ void AccessibilityHandler::notifyAccessibilityEvent (AccessibilityEvent eventTyp
             case AccessibilityEvent::textChanged:
             case AccessibilityEvent::valueChanged:          return NSAccessibilityValueChangedNotification;
             case AccessibilityEvent::titleChanged:          return NSAccessibilityTitleChangedNotification;
-            case AccessibilityEvent::structureChanged:      return layoutChangedNotification();
+            case AccessibilityEvent::structureChanged:      return NSAccessibilityLayoutChangedNotification;
         }
 
         return NSAccessibilityNotificationName{};
@@ -939,31 +915,28 @@ void AccessibilityHandler::postAnnouncement (const String& announcementString, A
     if (! areAnyAccessibilityClientsActive())
         return;
 
-    if (@available (macOS 10.9, *))
+    auto nsPriority = [priority]
     {
-        auto nsPriority = [priority]
+        // The below doesn't get noticed by the @available check above
+        JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wunguarded-availability")
+
+        switch (priority)
         {
-            // The below doesn't get noticed by the @available check above
-            JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wunguarded-availability")
+            case AnnouncementPriority::low:    return NSAccessibilityPriorityLow;
+            case AnnouncementPriority::medium: return NSAccessibilityPriorityMedium;
+            case AnnouncementPriority::high:   return NSAccessibilityPriorityHigh;
+        }
 
-            switch (priority)
-            {
-                case AnnouncementPriority::low:    return NSAccessibilityPriorityLow;
-                case AnnouncementPriority::medium: return NSAccessibilityPriorityMedium;
-                case AnnouncementPriority::high:   return NSAccessibilityPriorityHigh;
-            }
+        jassertfalse;
+        return NSAccessibilityPriorityLow;
 
-            jassertfalse;
-            return NSAccessibilityPriorityLow;
+        JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+    }();
 
-            JUCE_END_IGNORE_WARNINGS_GCC_LIKE
-        }();
-
-        sendAccessibilityEvent (static_cast<id> ([NSApp mainWindow]),
-                                NSAccessibilityAnnouncementRequestedNotification,
-                                @{ NSAccessibilityAnnouncementKey: juceStringToNS (announcementString),
-                                   NSAccessibilityPriorityKey:     @(nsPriority) });
-     }
+    sendAccessibilityEvent (static_cast<id> ([NSApp mainWindow]),
+                            NSAccessibilityAnnouncementRequestedNotification,
+                            @{ NSAccessibilityAnnouncementKey: juceStringToNS (announcementString),
+                               NSAccessibilityPriorityKey:     @(nsPriority) });
 }
 
 } // namespace juce
