@@ -32,6 +32,11 @@
   ==============================================================================
 */
 
+// This type isn't in the headers until v2.36
+#if ! WEBKIT_CHECK_VERSION (2, 36, 0)
+struct WebKitURISchemeResponse;
+#endif
+
 namespace juce
 {
 
@@ -1222,7 +1227,7 @@ public:
     Platform (WebBrowserComponent& browserIn,
               const WebBrowserComponent::Options& optionsIn,
               const StringArray& userStrings)
-        : Thread ("Webview"), browser (browserIn), userAgent (optionsIn.getUserAgent())
+        : Thread (SystemStats::getJUCEVersion() + ": Webview"), browser (browserIn), userAgent (optionsIn.getUserAgent())
     {
         webKitIsAvailable = WebKitSymbols::getInstance()->isWebKitAvailable();
         init (InitialisationData { optionsIn.getNativeIntegrationsEnabled(),
