@@ -474,7 +474,7 @@ public:
         }
         else
         {
-            if (owner.unloadPageWhenHidden && ! owner.blankPageShown)
+            if (webView != nullptr && owner.unloadPageWhenHidden && ! owner.blankPageShown)
             {
                 // when the component becomes invisible, some stuff like flash
                 // carries on playing audio, so we need to force it onto a blank
@@ -491,11 +491,10 @@ public:
 
     void fallbackPaint (Graphics& webBrowserComponentContext) override
     {
+        webBrowserComponentContext.fillAll (Colours::white);
+
         if (! hasBrowserBeenCreated())
-        {
-            webBrowserComponentContext.fillAll (Colours::white);
             checkWindowAssociation();
-        }
     }
 
     void focusGainedWithDirection (FocusChangeType, FocusChangeDirection direction) override
@@ -520,6 +519,12 @@ public:
 
     ~WebView2() override
     {
+        if (webView2ConstructionHelper.webView2BeingCreated == this)
+            webView2ConstructionHelper.webView2BeingCreated = nullptr;
+
+        webView2ConstructionHelper.viewsWaitingForCreation.erase (this);
+
+        cancelPendingUpdate();
         removeEventHandlers();
         closeWebView();
     }
@@ -801,7 +806,29 @@ private:
                             auto errorString = "Error code: " + String (errorStatus);
 
                             if (owner.pageLoadHadNetworkError (errorString))
-                                owner.goToURL ("data:text/plain;charset=UTF-8," + errorString);
+                            {
+                                const auto adhocErrorPageUrl = "data:text/plain;charset=UTF-8," + errorString;
+
+                                if (owner.lastURL == adhocErrorPageUrl)
+                                {
+                                    // We encountered an error while trying to navigate to the adhoc
+                                    // error page. Trying to navigate to the error page again would
+                                    // likely end us up in an infinite error callback loop, so we
+                                    // early exit.
+                                    //
+                                    // Override WebBrowserComponent::pageLoadHadNetworkError and
+                                    // return false to avoid such a loop, while still being able to
+                                    // take action on the error if necessary.
+                                    //
+                                    // Receiving Error code: 9 can often be ignored safely with the
+                                    // current WebView2 implementation.
+                                    jassertfalse;
+
+                                    return S_OK;
+                                }
+
+                                owner.goToURL (adhocErrorPageUrl);
+                            }
                         }
                     }
 
@@ -969,6 +996,8 @@ private:
 
     void setWebViewPreferences()
     {
+        setControlVisible (owner.isShowing());
+
         ComSmartPtr<ICoreWebView2Controller2> controller2;
         webViewController->QueryInterface (controller2.resetAndGetPointerAddress());
 
@@ -1023,8 +1052,6 @@ private:
             webView2ConstructionHelper.viewsWaitingForCreation.erase (this);
             webView2ConstructionHelper.webView2BeingCreated = this;
 
-            WeakReference<WebView2> weakThis (this);
-
             webViewHandle.environment->CreateCoreWebView2Controller ((HWND) peer->getNativeHandle(),
                 Callback<ICoreWebView2CreateCoreWebView2ControllerCompletedHandler> (
                     [weakThis = WeakReference<WebView2> { this }] (HRESULT, ICoreWebView2Controller* controller) -> HRESULT
